// #autoload
// #name = _ZFlagEvent
// #category = ZephyrFox's Scripts
// #description = Modifies the way flag events are handled.
// #version = 1.2
// #date = 11.26.2003
// #status = testing
// #author = Anthony Vrabel
// #warrior = ZephyrFox
// #email = ZephyrFox@hotmail.com
// #web = www.zephyrfox.com
// #credits = ProPack authors for the peakspeed idea, various other flag-call script authors for their ideas.
// #readme = _ZFlagEvent.txt
// #config = ZFlagEventOPTGUI
// #include = support/callback.cs
// #include = support/mission_callbacks.cs
// #include = support/loadout.cs
// #include = support/team_tracker.cs
// #include = support/mute_tools.cs

if(!isObject(ZFE))
{
	new ScriptObject(ZFE)
	{
		class 	= ZFE;
		version = "1.2";
	};
}

package ZFEoverloads
{
	function PlayGui::onWake(%this)
	{
		parent::onWake(%this);

		updatePeakSpeed();
	}
};
if(isActivePackage(ZFEoverloads)) deactivatePackage(ZFEoverloads);
activatePackage(ZFEoverloads);

function ZFE::setup(%this)
{
	callback.add(CallbackMSGCTFFlagTaken, "ZFE.flagTaken");
	callback.add(CallbackMSGCTFFlagDropped, "ZFE.flagDropped");
	callback.add(CallbackMSGCTFFlagCapped, "ZFE.flagCapped");
	callback.add(CallbackMSGCTFFlagReturned, "ZFE.flagReturned");

	callback.add(CallbackMSGCTFEnemyCap, "ZFE.enemyCap");
	callback.add(CallbackMSGCTFFriendCap, "ZFE.friendCap");

	callback.add(CallbackMSGCTFEnemyFlagTouch, "ZFE.enemyFlagTouch");
	callback.add(CallbackMSGCTFFriendFlagTouch, "ZFE.friendFlagTouch");

	callback.add(CallbackMSGCarKill, "ZFE.FCKill");

	callback.add(grenadeUsed, "ZFE.grenadeUsed");
	callback.add(RepairKitUsed, "ZFE.repKitUsed");
	callback.add(BeaconUsed, "ZFE.beaconUsed");

	callback.add(onGameOver, "ZFE.cleanup");
	callback.add(onUserClientDrop, "ZFE.cleanup");

	callback.add(CallbackLeaveMissionArea, "ZFE.leaveArea");

	// Audio
	new AudioProfile(playOnFlagGrab)
	{
		filename	= "Zeph/flagGrab.wav";
		description	= "audioGui";
		preload		= true;
	};

	new AudioProfile(playOnFlagCap)
	{
		filename	= "Zeph/flagCap.wav";
		description	= "audioGui";
		preload		= true;
	};

	new AudioProfile(playOnFlagDrop)
	{
		filename	= "Zeph/flagDrop.wav";
		description	= "audioGui";
		preload		= true;
	};

	new AudioProfile(playOnFlagReturn)
	{
		filename	= "Zeph/flagReturn.wav";
		description	= "audioGui";
		preload		= true;
	};

	new AudioProfile(playOnFCKill)
	{
		filename	= "Zeph/FCKill.wav";
		description	= "audioGui";
		preload		= true;
	};
}

function updatePeakSpeed()
{
	if(isEventPending($pkSchedule)) cancel($pkSchedule);

	%speed = getControlObjectSpeed() * 3.6;

	if (%speed == 0) $pkSpeed = 0;
	else if ((%speed > $pkSpeed)) $pkSpeed = %speed;

	$pkSchedule = schedule(250, 0, updatePeakSpeed);
}

function ZFE::flagTaken(%this, %msgType, %msgString, %clientName, %flagTeam, %flagTeamId, %clientNameBase)
{
	if (deTag(%clientName) $= teamTracker.myName && $ZFE::flagTaken)
	{
		%gspeed = getControlObjectSpeed() * 3.6;

		if(loadout.getGrenadeType() $= "Flare Grenade") %flareCount = loadout.getGrenadeAmmo();
		else %flareCount = 0;

		$gotFlag = true;
		$ZephGrabTime = ZFE.getTime();
		$ZephHelp = schedule(3000, 0, "commandToServer", 'CannedChat', 'ChatHelp', false);

		alxPlay(playOnFlagGrab, 0, 0, 0);

		if(loadout.getRepairKit() > 0) %message = "\c4Capper message: \c5Grabbed at \c1[" @ %gspeed @ " kph] \c5APPROACH peak \c1[" @ $pkSpeed @ " kph] \c5Flares Remaining \c1[" @ %flareCount @ "] \c5Repair Kit? \c1[YES]~wflg.flag";
			else %message = "\c4Capper message: \c5Grabbed at \c1[" @ %gspeed @ " kph] \c5APPROACH peak \c1[" @ $pkSpeed @ " kph] \c5Flares Remaining \c1[" @ %flareCount @ "] \c5Repair Kit? \c1[NO]~wflg.flag";
		commandToServer('TeamMessageSent', %message);	

		$pkSpeed = 0;
	}
}

function ZFE::flagCapped(%this, %msgType, %msgString, %clientName, %flagTeam, %flagTeamId, %clientNameBase)
{
	if(teamTracker.getPlayerTeamID(detag(%clientName),1) $= teamTracker.friendlyTeamID) $WEgotflag = false;
	else $NMEgotflag = false;

	if(deTag(%clientName) $= teamTracker.myName && $ZFE::flagCapped)
	{
		%cspeed = getControlObjectSpeed() * 3.6;

		$gotFlag = false;
		$ZephCapTime = ZFE.getTime();
		%time = ($ZephCapTime-$ZephGrabTime)/1000;
		cancel($ZephHelp);

		alxPlay(playOnFlagCap, 0, 0, 0);

		%message = "\c4Capper message: \c5Capped at \c1[" @ %cspeed @ " kph] \c5in \c1[" @ %time @ " seconds] \c5RETURN peak \c1[" @ $pkSpeed @ " kph]";
		commandToServer('TeamMessageSent', %message);

		$pkSpeed = 0;
	}
}

function ZFE::flagDropped(%this, %msgType, %msgString, %clientName, %flagTeam, %flagTeamId, %clientNameBase)
{
	if(teamTracker.getPlayerTeamID(detag(%clientName),1) $= teamTracker.friendlyTeamID) $WEgotflag = false;
	else $NMEgotflag = false;

	if(deTag(%clientName) $= teamTracker.myName && $ZFE::flagDropped)
	{
		%dspeed = getControlObjectSpeed() * 3.6;

		$gotFlag = false;
		cancel($ZephHelp);

		alxPlay(playOnFlagDrop, 0, 0, 0);

		if(%dspeed > 0) %message = "\c4Capper message: \c5Flag passed!!!  Speed at pass \c1[" @ %dspeed @ " kph]~watt.flag";
			else %message = "\c4Capper message: \c5Get the NME flag!!!~watt.flag";
		commandToServer('TeamMessageSent', %message);

		$pkSpeed = 0;
	}
}

function ZFE::flagReturned(%this, %msgType, %msgString, %clientName, %flagTeam, %flagTeamId, %clientNameBase)
{
	if(deTag(%clientName) $= teamTracker.myName && $ZFE::flagReturned)
	{
		%rspeed = getControlObjectSpeed() * 3.6;
		$ZephRetTime = ZFE.getTime();
		%time = ($ZephRetTime-$NMEgrabTime)/1000;

		alxPlay(playOnFlagReturn, 0, 0, 0);

		%message = "\c4LD message: \c5Returned at \c1[" @ %rspeed @ " kph] \c5within \c1[" @ %time @ " seconds] \c5of NME grab. RETRIEVE peak \c1[" @ $pkSpeed @ " kph]~wflg.secure";
		commandToServer('TeamMessageSent', %message);

		$pkSpeed = 0;
	}
}

function ZFE::FCKill(%this, %msgType, %msgString, %killerName)
{
	if(Strstr(%msgString, "You received") != -1 && $ZFE::FCKilled)
	{
		%fckspeed = getControlObjectSpeed() * 3.6;
		$ZephFCKTime = ZFE.getTime();
		%time = ($ZephFCKTime-$NMEgrabTime)/1000;

		alxPlay(playOnFCKill, 0, 0, 0);

		%message = "\c4LD message: \c5NME FC killed at \c1[" @ %fckspeed @ " kph] \c5within \c1[" @ %time @ " seconds] \c5of NME grab. INTERCEPT peak \c1[" @ $pkSpeed @ " kph]~wflg.retrieve";
		commandToServer('TeamMessageSent', %message);

		$pkSpeed = 0;
	}
}

function ZFE::enemyFlagTouch(%this, %msgType, %msgString, %toucher, %score, %plural)
{
	$NMEgotFlag = true;
	$NMEgrabTime = ZFE.getTime();
}

function ZFE::friendFlagTouch(%this, %msgType, %msgString, %toucher, %score, %plural)
{
	$WEgotFlag = true;
	$WEgrabTime = ZFE.getTime();
}

function ZFE::enemyCap(%this, %msgType, %msgString, %capper, %score, %plural)
{

}

function ZFE::friendCap(%this, %msgType, %msgString, %score, %plural)
{

}

function ZFE::grenadeUsed(%this)
{
	if(loadout.getGrenadeType() $= "Flare Grenade") $ZFEflareCount = loadout.getGrenadeAmmo();
	else $ZFEflareCount = 0;

	if($ZFEflareCount == 0 && $ZFErepkits == 0 && $ZFEbeacons == 0) return;

	if($gotFlag && $ZFE::FCflares)
	{
		if($ZFEflareCount > 0)
		{
			%message = "\c4Capper message: \c5Flares remaining \c1[" @ $ZFEflareCount @ "]";
			commandToServer('TeamMessageSent', %message);
		}
		else
		{
			%message = "\c4Capper message: \c5*** Flared out! ***";
			commandToServer('TeamMessageSent', %message);
		}
	}
}

function ZFE::repKitUsed(%this)
{
	$ZFErepkits = loadout.getRepairKit();

	if($ZFEflareCount == 0 && $ZFErepkits == 0 && $ZFEbeacons == 0) return;

	if($gotFlag && $ZFE::FCrepkit)
	{
		%message = "\c4Capper message: \c5Repair kit used!";
		commandToServer('TeamMessageSent', %message);
	}
}

function ZFE::beaconUsed(%this)
{
	$ZFEbeacons = loadout.getBeaconAmmo();

	if($ZFEflareCount == 0 && $ZFErepkits == 0 && $ZFEbeacons == 0) return;

	if($ZFEbeacons == 2 && $ZFE::beaconStartRun && !$gotFlag && MissionCallback.getMissionType() $= "Capture the Flag")
	{
		%message = "\c4Capper message: \c5*** IN POSITION ***~wtgt.flag";
		commandToServer('TeamMessageSent', %message);
	}
	else if($ZFEbeacons == 1 && $ZFE::beaconStartRun &&  !$gotFlag && MissionCallback.getMissionType() $= "Capture the Flag")
	{
		%message = "\c4Capper message: \c5*** CEASE FIRE ***~wvqk.ceasefire";
		commandToServer('TeamMessageSent', %message);
	}
	else if($ZFEbeacons == 0 && $ZFE::beaconStartRun && !$gotFlag && MissionCallback.getMissionType() $= "Capture the Flag")
	{
		%message = "\c4Capper message: \c5*** BEGINNING RUN ***~wslf.att.flag";
		commandToServer('TeamMessageSent', %message);
		schedule(2000, 0, "commandToServer", 'CannedChat', 'ChatCmdDefendMe', false);
	}
}

function ZFE::leaveArea(%this)
{
	$gotFlag = false;
}

function ZFE::cleanup(%this)
{
	$WEgotFlag = false;
	$NMEgotFlag = false;
	$gotFlag = false;
	$ZFEbeacons = 0;
	$ZFErepkits = 0;
	$ZFEflarecount = 0;
}

function ZFE::getTime(%this)
{
	return mFloor(getSimTime());
}

function ZFE::loadPrefs()
{
	if(!$ZFE::loaded)
	{
		exec("prefs/ZFE.prefs");
		$ZFE::loaded = 1;
	}
}

function ZFE::savePrefs()
{
	export("$ZFE::*", "prefs/ZFE.prefs", false);
}

ZFE.loadPrefs();
ZFE.setup();

function ZFlagEventOPTGUI::onWake(%this)
{
	%Hres = getWord($pref::Video::resolution, 0);

	switch(%Hres)
	{
		case 1600:	%HGUIext = 1360;
				%VGUIext = 1073;

				%HPANext = 1340;
				%VPANext = 1053;

				%HSCRext = 1310;
				%VSCRext = 1013;

				%HCONext = 1285;
				%VCONext = 988;

		case 1280:	%HGUIext = 1040;
				%VGUIext = 897;

				%HPANext = 1020;
				%VPANext = 877;

				%HSCRext = 990;
				%VSCRext = 837;

				%HCONext = 965;
				%VCONext = 812;

		case 1024:	%HGUIext = 784;
				%VGUIext = 641;

				%HPANext = 764;
				%VPANext = 621;

				%HSCRext = 734;
				%VSCRext = 581;

				%HCONext = 709;
				%VCONext = 556;

		case 800:	%HGUIext = 560;
				%VGUIext = 473;

				%HPANext = 540;
				%VPANext = 453;

				%HSCRext = 510;
				%VSCRext = 413;

				%HCONext = 485;
				%VCONext = 388;

		default:	%HGUIext = 784;
				%VGUIext = 641;

				%HPANext = 764;
				%VPANext = 621;

				%HSCRext = 734;
				%VSCRext = 581;

				%HCONext = 709;
				%VCONext = 556;
	}

	ZFlagEventOPTGUI.setExtent(%HGUIext, %VGUIext);
	ZFlagEventOPTPane.setExtent(%HPANext, %VPANext);
	ZFlagEventOPTScroll.setExtent(%HSCRext, %VSCRext);
	ZFlagEventOPTContent.setExtent(%HCONext, %VCONext);
}

//--- OBJECT WRITE BEGIN ---
new GuiControl(ZFlagEventOPTGUI) {
	profile = "GuiDefaultProfile";
	horizSizing = "right";
	vertSizing = "bottom";
	position = "5 12";
	extent = "0 0";
	minExtent = "8 8";
	visible = "1";
	hideCursor = "0";
	bypassHideCursor = "0";
	helpTag = "0";

	new ShellPaneCtrl(ZFlagEventOPTPane) {
		profile = "ShellPaneProfile";
		horizSizing = "right";
		vertSizing = "bottom";
		position = "0 0";
		extent = "0 0";
		minExtent = "48 92";
		visible = "1";
		hideCursor = "0";
		bypassHideCursor = "0";
		helpTag = "0";
		text = "ZFlagEvent Options";
		longTextBuffer = "0";
		maxLength = "255";
		noTitleBar = "0";
	};
	new ShellScrollCtrl(ZFlagEventOPTScroll) {
		profile = "NewScrollCtrlProfile";
		horizSizing = "right";
		vertSizing = "bottom";
		position = "15 25";
		extent = "0 0";
		minExtent = "24 24";
		visible = "1";
		hideCursor = "0";
		bypassHideCursor = "0";
		helpTag = "0";
		willFirstRespond = "1";
		hScrollBar = "dynamic";
		vScrollBar = "dynamic";
		constantThumbHeight = "0";
		defaultLineHeight = "15";
		childMargin = "0 0";
		fieldBase = "gui/shll_field";

		new GuiScrollContentCtrl(ZFlagEventOPTContent) {
			profile = "GuiDefaultProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "4 4";
			extent = "0 0";
			minExtent = "8 8";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			helpTag = "0";

			new GuiControl(ZFlagEventOPTCanvas) {
				profile = "GuiDefaultProfile";
				horizSizing = "right";
				vertSizing = "bottom";
				position = "0 0";
				extent = "1280 1024";
				minExtent = "8 8";
				visible = "1";
				hideCursor = "0";
				bypassHideCursor = "0";
				helpTag = "0";

				new ShellToggleButton() {
					profile = "ShellRadioProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "10 20";
					extent = "170 30";
					minExtent = "26 27";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					variable = "$ZFE::flagTaken";
					helpTag = "0";
					text = "Enable flag grab message";
					longTextBuffer = "0";
					maxLength = "255";
				};
				new ShellToggleButton() {
					profile = "ShellRadioProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "10 45";
					extent = "170 30";
					minExtent = "26 27";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					variable = "$ZFE::flagCapped";
					helpTag = "0";
					text = "Enable flag cap message";
					longTextBuffer = "0";
					maxLength = "255";
				};
				new ShellToggleButton() {
					profile = "ShellRadioProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "10 70";
					extent = "170 30";
					minExtent = "26 27";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					variable = "$ZFE::flagDropped";
					helpTag = "0";
					text = "Enable flag dropped message";
					longTextBuffer = "0";
					maxLength = "255";
				};
				new ShellToggleButton() {
					profile = "ShellRadioProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "10 95";
					extent = "170 30";
					minExtent = "26 27";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					variable = "$ZFE::flagReturned";
					helpTag = "0";
					text = "Enable flag return message";
					longTextBuffer = "0";
					maxLength = "255";
				};
				new ShellToggleButton() {
					profile = "ShellRadioProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "10 120";
					extent = "170 30";
					minExtent = "26 27";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					variable = "$ZFE::FCkilled";
					helpTag = "0";
					text = "Enable FC killed message";
					longTextBuffer = "0";
					maxLength = "255";
				};
				new ShellToggleButton() {
					profile = "ShellRadioProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "10 145";
					extent = "170 30";
					minExtent = "26 27";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					variable = "$ZFE::FCflares";
					helpTag = "0";
					text = "Enable flare messages";
					longTextBuffer = "0";
					maxLength = "255";
				};
				new ShellToggleButton() {
					profile = "ShellRadioProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "10 170";
					extent = "170 30";
					minExtent = "26 27";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					variable = "$ZFE::FCrepkit";
					helpTag = "0";
					text = "Enable repkit messages";
					longTextBuffer = "0";
					maxLength = "255";
				};
				new ShellToggleButton() {
					profile = "ShellRadioProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "10 195";
					extent = "170 30";
					minExtent = "26 27";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					variable = "$ZFE::beaconStartRun";
					helpTag = "0";
					text = "Enable beacon calling";
					longTextBuffer = "0";
					maxLength = "255";
				};
				new ShellBitmapButton() {
					profile = "ShellButtonProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "7 220";
					extent = "100 38";
					minExtent = "32 38";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					command = "ZFE.savePrefs();";
					helpTag = "0";
					text = "Save";
					simpleStyle = "0";
				};
			};
		};
	};
};
//--- OBJECT WRITE END ---